from typing import Optional
from raga import Dataset, StringElement, AggregationLevelElement, ModelABTestRules, ModelABTestTypeElement

def model_ab_test(test_ds: Dataset, testName: StringElement, modelA: StringElement, modelB: StringElement,
                  type: ModelABTestTypeElement, aggregation_level: AggregationLevelElement, rules: ModelABTestRules,
                  gt: Optional[StringElement] = StringElement(""), filter: Optional[StringElement] = StringElement("")):
    ab_test_validation(test_ds, testName, modelA, modelB, type, aggregation_level, rules, gt)      
    return {
            "datasetId": test_ds.dataset_id,
            "experimentId": test_ds.test_session.experiment_id,
            "name": testName.get(),
            "filter": filter.get(),
            "modelA": modelA.get(),
            "modelB": modelB.get(),
            "gt": gt.get(),
            "type": type.get(),
            "aggregationLevels": aggregation_level.get(),
            "rules": rules.get()
        }


def ab_test_validation(test_ds: Dataset, testName: StringElement, modelA: StringElement, modelB: StringElement,
               type: ModelABTestTypeElement, aggregation_level: AggregationLevelElement, rules: ModelABTestRules,
               gt: Optional[StringElement] = StringElement("")):
    assert isinstance(test_ds, Dataset), "test_ds must be an instance of the Dataset class."
    assert isinstance(testName, StringElement) and testName.get(), "testName is required and must be an instance of the StringElement class."
    assert isinstance(modelA, StringElement) and modelA.get(), "modelA is required and must be an instance of the StringElement class."
    assert isinstance(modelB, StringElement) and modelB.get(), "modelB is required and must be an instance of the StringElement class."
    assert isinstance(type, ModelABTestTypeElement), "type must be an instance of the ModelABTestTypeElement class."
    assert isinstance(aggregation_level, AggregationLevelElement) and aggregation_level.get(), "aggregation_level is required and must be an instance of the AggregationLevelElement class."
    assert isinstance(rules, ModelABTestRules) and rules.get(), "rules is required and must be an instance of the ModelABTestRules class."

    if type.get() == "labelled":
        assert isinstance(gt, StringElement) and gt.get(), "gt is required on labelled type and must be an instance of the StringElement class."

    if type.get() == "unlabelled":
        if isinstance(gt, StringElement) and gt.get():
            raise ValueError("gt is not required on unlabelled type.")
    return True




class TestSessionValidator:
    @staticmethod
    def validate_project_name(project_name: str) -> str:
        if not project_name:
            raise ValueError("Project name is required.")
        
        # Check if project_name contains any special characters except "_"
        allowed_chars = set("._-abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 ")

        if not set(project_name).issubset(allowed_chars):
            raise ValueError("Project name should only contain alphanumeric characters and '_', '.'.")
        # Additional validation logic specific to project id
        return project_name

    @staticmethod
    def validate_run_name(run_name: str) -> str:
        if not run_name:
            raise ValueError("Run name is required.")
        # Check if run_name contains any special characters except "_"
        allowed_chars = set("._-abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 ")

        if not set(run_name).issubset(allowed_chars):
            raise ValueError("Run name should only contain alphanumeric characters and '_', '.'.")
        # Additional validation logic specific to run name
        return run_name





import json


json_files = {
    "baseline_rtl":["baseline1-re.json", "baseline2-re.json", "baseline3-re.json"],
    "improved_rtl":["improved1-re.json", "improved2-re.json", "improved3-re.json"],
}


for models, files in json_files.items():
    json_file_name = f"{models}.json"
    json_file_data = []
    for file in files:
        with open(file, "r") as f:
            json_data = json.load(f)
            json_file_data.extend(json_data)
    with open(json_file_name, "w") as write_f:
        json.dump(json_file_data, write_f, indent=4)

print("JSON files created successfully.")


from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(json_file_path):
    test_data_frame = []
    with open(json_file_path, 'r') as json_file:
        # Load JSON data
        json_data = json.load(json_file)
        hr = 1
        # Process the JSON data
        for item in json_data:
            AnnotationsV1 = ImageDetectionObject()
            ROIVectorsM1 = ROIEmbedding()
            ImageVectorsM1 = ImageEmbedding()
            for detection in item["outputs"][0]["detections"]:
                AnnotationsV1.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
                for roi_emb in detection['roi_embedding']:
                    ROIVectorsM1.add(Embedding(roi_emb))
            
            attributes_dict = {}
            attributes = item.get("attributes", {})
            for key, value in attributes.items():
                attributes_dict[key] = StringElement(value)

            image_embeddings = item.get("image_embedding", {})
            for value in image_embeddings:
                ImageVectorsM1.add(Embedding(value))

            data_point = {
                'ImageId': StringElement(item["inputs"][0]),
                'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
                'SourceLink': StringElement(item["inputs"][0]),
                'AnnotationsV1': AnnotationsV1,
                'ROIVectorsM1': ROIVectorsM1,
                'ImageVectorsM1': ImageVectorsM1,
            }

            merged_dict = {**data_point, **attributes_dict}
            test_data_frame.append(merged_dict)
            hr+=1
    return test_data_frame

pd_modelA = pd.DataFrame(convert_json_to_data_frame("test-dataset-modelA.json"))
pd_modelB = pd.DataFrame(convert_json_to_data_frame("test-dataset-modelB.json"))


schema_model_a = RagaSchema()
schema_model_a.add("ImageId", PredictionSchemaElement(), pd_modelA)
schema_model_a.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_modelA)
schema_model_a.add("SourceLink", FeatureSchemaElement(), pd_modelA)
schema_model_a.add("Reflection", AttributeSchemaElement(), pd_modelA)
schema_model_a.add("Overlap", AttributeSchemaElement(), pd_modelA)
schema_model_a.add("CameraAngle", AttributeSchemaElement(), pd_modelA)
schema_model_a.add("AnnotationsV1", InferenceSchemaElement(model="modelA"), pd_modelA)
schema_model_a.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="modelA", ref_col_name=""), pd_modelA)
schema_model_a.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="modelA", ref_col_name=""), pd_modelA)

schema_model_b = RagaSchema()
schema_model_b.add("ImageId", PredictionSchemaElement(), pd_modelB)
schema_model_b.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_modelB)
schema_model_b.add("SourceLink", FeatureSchemaElement(), pd_modelB)
schema_model_b.add("Reflection", AttributeSchemaElement(), pd_modelB)
schema_model_b.add("Overlap", AttributeSchemaElement(), pd_modelB)
schema_model_b.add("CameraAngle", AttributeSchemaElement(), pd_modelB)
schema_model_b.add("AnnotationsV1", InferenceSchemaElement(model="ModelB"), pd_modelB)
schema_model_b.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="ModelB", ref_col_name=""), pd_modelB)
schema_model_b.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="ModelB", ref_col_name=""), pd_modelB)


test_session = TestSession(project_name="testingProject",run_name= "test_iteration-test-exp-2")

test_ds = Dataset(test_session=test_session, name="test-dataset-name")

test_ds.load(pd_modelA, schema_model_a)
test_ds.load(pd_modelB, schema_model_b)